package models

import (
	"go.mongodb.org/mongo-driver/bson"
	"go.mongodb.org/mongo-driver/mongo"
	ct "gogs.mikescher.com/BlackForestBytes/goext/cursortoken"
)

type BlobFilter struct {
	Reference *AnyID
	Type      *BlobType
	Creator   *AnyID
	MimeType  *MimeType
}

func (f BlobFilter) FilterQuery() mongo.Pipeline {
	pipeline := mongo.Pipeline{}

	pipeline = append(pipeline, bson.D{{Key: "$match", Value: bson.M{"deleted": nil}}})

	if f.Type != nil {
		pipeline = append(pipeline, bson.D{{Key: "$match", Value: bson.M{"type": *f.Type}}})
	}

	if f.MimeType != nil {
		pipeline = append(pipeline, bson.D{{Key: "$match", Value: bson.M{"mimeType": *f.MimeType}}})
	}

	if f.Creator != nil {
		pipeline = append(pipeline, bson.D{{Key: "$match", Value: bson.M{"creator": *f.Creator}}})
	}

	if f.Reference != nil {
		pipeline = append(pipeline, bson.D{{Key: "$match", Value: bson.M{"reference": *f.Reference}}})
	}

	return pipeline
}

func (f BlobFilter) Pagination() (string, ct.SortDirection, string, ct.SortDirection) {
	return "creationTime", ct.SortDESC, "_id", ct.SortDESC
}
