package models

import "strings"

type MimeCategory string //@enum:type

const (
	CatDocument  MimeCategory = "DOCUMENT"
	CatImage     MimeCategory = "IMAGE"
	CatContainer MimeCategory = "CONTAINER"
)

type MimeType string
type mimespec struct {
	Mime      MimeType
	Extension string
	Category  MimeCategory
}

var mimeSpecs = []mimespec{
	{"application/pdf", "pdf", CatDocument},
	{"image/jpeg", "jpg", CatImage},
	{"image/jpeg", "jpeg", CatImage},
	{"image/png", "png", CatImage},
	{"image/gif", "gif", CatImage},
	{"image/tiff", "tiff", CatImage},
	{"image/webp", "webp", CatImage},
	{"application/msword", "doc", CatDocument},
	{"application/vnd.openxmlformats-officedocument.wordprocessingml.document", "docx", CatDocument},
	{"application/zip", "zip", CatContainer},
	{"application/csv", "csv", CatDocument},
	{"application/html", "html", CatDocument},
	{"application/vnd.ms-powerpoint", "ppt", CatDocument},
	{"application/vnd.openxmlformats-officedocument.presentationml.presentation", "pptx", CatDocument},
	{"image/svg+xml", "svg", CatImage},
	{"application/x-tar", "tar", CatContainer},
	{"text/plain", "txt", CatDocument},
	{"text/markdown", "md", CatDocument},
	{"text/markdown", "markdown", CatDocument},
	{"text/markdown", "mdown", CatDocument},
	{"text/markdown", "markdn", CatDocument},
	{"application/vnd.ms-excel", "xls", CatDocument},
	{"application/vnd.openxmlformats-officedocument.spreadsheetml.sheet", "xlsx", CatDocument},
}

func (ct MimeType) Extension() string {
	for _, v := range mimeSpecs {
		if v.Mime == ct {
			return v.Extension
		}
	}

	return "bin"
}

func (ct MimeType) String() string {
	return string(ct)
}

func (ct MimeType) spec() *mimespec {
	for _, v := range mimeSpecs {
		if v.Mime == ct {
			return &v
		}
	}
	return nil
}

func (ct MimeType) IsValid() bool {
	return ct.spec() != nil
}

func (ct MimeType) IsImage() bool {
	if spec := ct.spec(); spec != nil {
		return spec.Category == CatImage
	}
	return false
}

func DetermineMimeFromFilename(fn string) MimeType {
	fn = strings.ToLower(fn)
	for _, v := range mimeSpecs {
		if strings.HasSuffix(fn, "."+v.Extension) {
			return v.Mime
		}
	}
	return ""
}
