package models

import (
	"go.mongodb.org/mongo-driver/bson"
	"go.mongodb.org/mongo-driver/mongo"
	ct "gogs.mikescher.com/BlackForestBytes/goext/cursortoken"
	"regexp"
)

type ConversationNoteFilter struct {
	OrganizationID *OrganizationID
	Search         *string
}

func (f ConversationNoteFilter) FilterQuery() mongo.Pipeline {
	pipeline := mongo.Pipeline{}

	pipeline = append(pipeline, bson.D{{Key: "$match", Value: bson.M{"deleted": nil}}})

	if f.OrganizationID != nil {
		pipeline = append(pipeline, bson.D{{Key: "$match", Value: bson.M{"organizationID": *f.OrganizationID}}})
	}

	if f.Search != nil {
		ss := ".*" + regexp.QuoteMeta(*f.Search) + ".*"
		pipeline = append(pipeline, bson.D{{Key: "$match", Value: bson.M{
			"$or": bson.A{
				bson.M{"title": bson.M{"$regex": ss, "$options": "i"}},
				bson.M{"body": bson.M{"$regex": ss, "$options": "i"}},
			},
		}}})
	}

	return pipeline
}

func (f ConversationNoteFilter) Pagination() (string, ct.SortDirection, string, ct.SortDirection) {
	return "creationTime", ct.SortASC, "_id", ct.SortASC
}
