package models

import (
	"go.mongodb.org/mongo-driver/bson"
	"go.mongodb.org/mongo-driver/mongo"
	ct "gogs.mikescher.com/BlackForestBytes/goext/cursortoken"
	"regexp"
)

type GroupFilter struct {
	Organization    *OrganizationID
	Search          *string
	FrontendEnabled *bool
}

func (f GroupFilter) FilterQuery() mongo.Pipeline {
	pipeline := mongo.Pipeline{}

	pipeline = append(pipeline, bson.D{{Key: "$match", Value: bson.M{"deleted": nil}}})

	if f.Organization != nil {
		pipeline = append(pipeline, bson.D{{Key: "$match", Value: bson.M{"organizationID": *f.Organization}}})
	}

	if f.FrontendEnabled != nil {
		pipeline = append(pipeline, bson.D{{Key: "$match", Value: bson.M{"frontendEnabled": *f.FrontendEnabled}}})
	}

	if f.Search != nil {
		ss := ".*" + regexp.QuoteMeta(*f.Search) + ".*"
		pipeline = append(pipeline, bson.D{{Key: "$match", Value: bson.M{
			"$or": bson.A{
				bson.M{"title": bson.M{"$regex": ss, "$options": "i"}},
			},
		}}})
	}

	return pipeline
}

func (f GroupFilter) Pagination() (string, ct.SortDirection, string, ct.SortDirection) {
	return "creationTime", ct.SortDESC, "_id", ct.SortDESC
}
